"""
Web Fragments Django application initialization.
"""
from django.apps import AppConfig


class WebFragmentsConfig(AppConfig):
    """
    Configuration for the Web Fragments Django application.
    """

    name = 'web_fragments'
